#!/usr/bin/env ruby
require 'base64'
require 'openssl'
require 'net/http'

# This class contains functions designed to send GET and POST
# requests to the Media Portal API.
#
# Attributes:
#  keyId: The string ID for the API Key you'll use to authorize your API calls with.
#  keySecret: The bytearray Secret for the API Key you'll use to authorize your API calls with.
class MediaPortalAPICommunicator
  attr_accessor :keyId
  attr_accessor :keySecret

  def initialize(keyId=nil, keySecret=nil)
    @keyId = keyId
    @keySecret = keySecret
  end

  # Computes the encoded authentication signature for a Media Portal API call.
  #
  # Args:
  #  dateStr: Date in "%a, %d %b %Y %H:%M:%S %Z" format.
  #  uri: Full URI of the service you wish to call (Not Including Parameters).
  #  contentType: The format for which POST body parameters will be sent from the client. Typically "application/xml" or "application/json" (send an empty string if this is for a GET request).
  #  method: The API method type ("GET," "POST," etc.) that you will be calling.
  #  md5: The MD5 Hashed value of your post body (send an empty string if this is for a GET request or post body is null).
  #
  # Returns:
  #  string encoded in base64 to be added to an API call for authentication
  def getAPISignature(dateStr, uri, contentType, method, md5)
    # The authentication signature is built as shown
    #     Mon, 01 Jan 2018 00:00:00 GMT\n
    #     /api/uri/{access group ID}/{scid}/{property}\n
    #     application/json\n
    #     GET\n
    #     MD5 Hash (if applicable)

    data = "#{dateStr}\n#{uri}\n#{contentType}\n#{method}\n#{md5}"
    puts "*****API Signature Pre-Hash*****\n#{data}\n**************End***************\n\n"

    hmac = OpenSSL::HMAC.digest 'sha1', @keySecret, data
    return Base64.encode64(hmac).to_s
  end

  #  Makes a GET service call to the Media Portal API.
  #  This demo includes information on how requests are built and sent, and includes
  #  information on creating authentication signatures (required in order to prove the
  #  user has necessary permissions to access the results).
  #
  # Args:
  #  accept: The format for which data from the service call should be returned. Typically "application/xml" or "application/json."
  #  dateStr: Date in "D, d M Y H:i:s T" format.
  #  url: Full URL of the service you wish to call (EX. https://fakeApiURL.com/services/key?param1=test1&param2=test2,test3).
  #
  # Returns:
  #  A String containing the message returned by the call.
  #
  # Raises:
  #  urllib.error.HTTPError: An error occurred while making the API call.
  def sendTestAPICallGET(url, accept="application/json", dateStr=Time.now.utc.strftime("%a, %d %b %Y %H:%M:%S" + "  GMT"))
    uri = URI.parse url
    signatureStr = getAPISignature dateStr, uri.path, "", "GET", ""
    authStr = "MPA #{@keyId}:#{signatureStr.strip}"
    puts "********Header Properties********"
    puts "Accept: #{accept}"
    puts "Authorization: #{authStr}"
    puts "Date: #{dateStr}"
    puts "Content-Length: 0"
    puts "Content-MD5: "
    puts "Content-Type: "
    puts "**************End****************\n"

    Net::HTTP.start(uri.host, uri.port,
      :use_ssl => uri.scheme == 'https') do |http|

      req = Net::HTTP::Get.new uri
      req["Accept"] =  accept
      req["Authorization"] =  authStr
      req["Date"] =  dateStr
      req["Content-Length"] =  0
      req["Content-MD5"] =  ""
      req["Content-Type"] =  ""
      res = http.request req
      return res.body
    end
  end

  # Makes a POST service call to the Media Portal API.
  # This demo includes information on how requests are built and sent, and includes
  # information on creating authentication signatures (required in order to prove the
  # user has necessary permissions to access the results).
  #
  # Args:
  #  accept: The format for which data from the service call should be returned. Typically "application/xml" or "application/json."
  #  contentType: The format for which POST body parameters will be sent from the client. Typically "application/xml" or "application/json."
  #  dateStr: Date in "D, d M Y H:i:s T" format.
  #  postBody: A string containing the JSON/XML POST request body parameter to be sent from the client.
  #  url: Full URL of the service you wish to call (EX. https://fakeApiURL.com/services/key?param1=test1&param2=test2,test3).
  #
  # Returns:
  #  A String containing the message returned by the call.
  #
  # Raises:
  #  urllib.error.HTTPError: An error occurred while making the API call.
  def sendTestAPICallPOST(url, accept="application/json", contentType="application/json", dateStr=Time.now.utc.strftime("%a, %d %b %Y %H:%M:%S GMT"), postBody="")
    data = postBody.encode Encoding::UTF_8
    md5 = Digest::MD5.hexdigest data
    uri = URI.parse url
    signatureStr = getAPISignature dateStr, uri.path, contentType, "POST", md5
    authStr = "MPA #{@keyId}:#{signatureStr.strip}"

    puts "********Header Properties********"
    puts "Accept: #{accept}"
    puts "Authorization: #{authStr}"
    puts "Date: #{dateStr}"
    puts "Content-Length: #{data.length}"
    puts "Content-MD5: #{md5}"
    puts "Content-Type: #{contentType}"
    puts "**************End****************\n"

    Net::HTTP.start(uri.host, uri.port,
      :use_ssl => uri.scheme == 'https') do |http|

      req = Net::HTTP::Post.new uri
      req["Accept"] =  accept
      req["Authorization"] =  authStr
      req["Date"] =  dateStr
      req["Content-Length"] =  data.length
      req["Content-MD5"] =  md5.to_s
      req["Content-Type"] =  contentType

      req.body = data

      res = http.request req # Net::HTTPResponse object
      return res.body
    end
  end
end
